@@ -6,6 +6,11 @@ Abstract representation of Atoms (:mod:`sknano.chemistry._atoms`)
 
 .. currentmodule:: sknano.chemistry._atoms
 
+.. todo::
+
+   Apply more consistent use of upper and lower case vars for `Atoms`
+   attributes.
+
 """
 from __future__ import division, absolute_import, print_function
 __docformat__ = 'restructuredtext'
@@ -17,42 +22,69 @@ from collections import OrderedDict, MutableSequence
 
 import numpy as np
 
+try:
+    from scipy.spatial import KDTree
+    has_kdtree = True
+except ImportError:
+    print('Install scipy version >= 0.13.0 to allow '
+          'nearest-neighbor queries between `Atoms`.')
+    has_kdtree = False
+
 from pkshared.tools.refdata import dimensions
 from ._atom import Atom
 
+__all__ = ['Atoms']
+
 
 class Atoms(MutableSequence):
 
-    """Class for creating abstract represention of a collection of Atoms.
+    """Class for creating abstract represention of a collection of `Atoms`.
 
     Parameters
     ----------
-    atoms : {None, sequence}, optional
-        if not ``None``, then a list of ``Atom`` objects
+    atoms : {None, sequence, `Atoms`}, optional
+        if not `None`, then a list of `Atom` instance objects or an
+        existing `Atoms` instance object.
     copylist : bool, optional
         perform shallow copy of atoms list
     deepcopy : bool, optional
         perform deepcopy of atoms list
+    use_kdtree : bool, optional
+        use :py:class:`~scipy:scipy.spatial.KDTree` to perform
+        nearest-neighbor analysis.
 
     """
 
-    def __init__(self, atoms=None, copylist=True, deepcopy=False):
+    def __init__(self, atoms=None, copylist=True, deepcopy=False,
+                 use_kdtree=True):
+
         self._atoms = []
+        self._atomtypes = {}
 
         self._atom_ids = []
         self._molecule_ids = []
+
         self._charges = []
         self._coords = []
+        #self._positions = []
         self._masses = []
         self._velocities = []
         self._symbols = []
-
-        self._atomtypes = {}
+        self._coordination_numbers = []
+        self._nearest_neighbors = []
 
         self._property_lists = \
             {'m': self._masses, 'q': self._charges, 'r': self._coords,
              'v': self._velocities, 'atomID': self._atom_ids,
-             'moleculeID': self._molecule_ids, 'symbol': self._symbols}
+             'moleculeID': self._molecule_ids, 'symbol': self._symbols,
+             'CN': self._coordination_numbers, 'NN': self._nearest_neighbors}
+
+        self._atom_tree = None
+
+        if use_kdtree and has_kdtree is False:
+            use_kdtree = False
+        self._use_kdtree = use_kdtree
+
         if atoms is not None:
             if isinstance(atoms, list):
                 if copylist and not deepcopy:
@@ -74,10 +106,14 @@ class Atoms(MutableSequence):
                 else:
                     self._atoms.extend(atoms.atoms)
             else:
-                raise TypeError('Atoms(atoms={!r}) '.format(atoms) +
-                                'is not a valid Atoms constructor argument.\n'
-                                'atoms must be None, a list of Atom objects, '
-                                'or an Atoms object instance.')
+                raise TypeError('`Atoms(atoms={!r})` '.format(atoms) +
+                                'is not a valid `Atoms` constructor '
+                                'argument.\n atoms must be `None`, a list '
+                                'of `Atom` objects, or an `Atoms` object '
+                                'instance.')
+
+            if use_kdtree:
+                self._atom_tree = self.atom_tree
 
     def _check_type(self, value):
         """Check that value is instance of `Atom` class.
@@ -103,24 +139,20 @@ class Atoms(MutableSequence):
             atoms_str += str(atom)
         return atoms_str
 
-    @property
-    def atom_ids(self):
-        """Return array of Atom IDs."""
-        atom_ids = []
-        for atom in self._atoms:
-            atom_ids.append(atom.atomID)
-        self._atom_ids = atom_ids[:]
-        return np.asarray(self._atom_ids)
-
     @property
     def atoms(self):
-        """Return the list of Atom objects"""
+        """Return the list of `Atom` objects"""
         return self._atoms
 
     @property
     def atomlist(self):
-        """Return the list of Atom objects"""
-        return self._atoms
+        """Return the list of `Atom` objects
+
+        .. deprecated:: 0.1.0
+           Use :py:attr:`atoms` attribute.
+
+        """
+        return self.atoms
 
     @property
     def atomtypes(self):
@@ -132,9 +164,25 @@ class Atoms(MutableSequence):
                 self._atomtypes[atom.atomtype]['q'] = atom.q
         return self._atomtypes
 
+    @property
+    def atom_ids(self):
+        """Return array of `Atom` IDs."""
+        atom_ids = []
+        for atom in self._atoms:
+            atom_ids.append(atom.atomID)
+        self._atom_ids = atom_ids[:]
+        return np.asarray(self._atom_ids)
+
+    @property
+    def atom_tree(self):
+        """Return the :py:class:`~scipy:scipy.spatial.KDTree` of coords."""
+        if self._use_kdtree and len(self._atoms) != 0:
+            self._atom_tree = KDTree(self.coords)
+        return self._atom_tree
+
     @property
     def charges(self):
-        """Return list of Atom charges."""
+        """Return list of `Atom` charges."""
         #self._charges_array = np.asarray(self._charges)
         charges = []
         for atom in self._atoms:
@@ -144,12 +192,12 @@ class Atoms(MutableSequence):
 
     @property
     def CM(self):
-        """Center-of-Mass coordinates of Atoms.
+        """Center-of-Mass coordinates of `Atoms`.
 
         Returns
         -------
         ndarray
-            3-element ndarray specifying center-of-mass coordinates of Atoms.
+            3-element ndarray specifying center-of-mass coordinates of `Atoms`.
 
         """
         masses = np.asarray([self.masses])
@@ -159,32 +207,118 @@ class Atoms(MutableSequence):
 
     @property
     def coords(self):
-        """Return array of Atom position array."""
-        #self._coords_array = np.asarray(self._coords)
+        """Return array of `Atom` coordinates."""
         coords = []
         for atom in self._atoms:
             coords.append(atom.r)
         self._coords = coords[:]
         return np.asarray(self._coords)
 
-    @property
-    def m(self):
-        """Total mass of all atoms.
+    #@property
+    #def positions(self):
+    #    """Return array of `Atom` positions."""
+    #    positions = []
+    #    for atom in self._atoms:
+    #        positions.append(atom.r)
+    #    self._positions = positions[:]
+    #    return np.asarray(self._positions)
 
-        .. deprecated:: 0.1.0
-           Use :py:attr:`M` property instead.
+    @property
+    def coordination_numbers(self):
+        """Return array of `Atom` coordination numbers."""
+        coordination_numbers = []
+        for atom in self._atoms:
+            coordination_numbers.append(atom.CN)
+        self._coordination_numbers = coordination_numbers[:]
+        return np.asarray(self._coordination_numbers)
+
+    def update_coordination_numbers(self, n=6, rc=np.inf, maxdiff=1):
+        """Update `Atom` coordination numbers.
+
+        Parameters
+        ----------
+        n : int, optional
+        rc : nonnegative float, optional
+        maxdiff : float, optional
 
         """
-        return self.M
+        if self._use_kdtree:
+            #if cutoff_intervals is None:
+            #    if cutoff_radius is None:
+            #        cutoff_radius = np.inf
+            #    cutoff_intervals = [cutoff_radius]
+
+            #    if isinstance(cutoff_intervals, list):
+            #        for rc in cutoff_intervals:
+            NN_d, NN_i = self.query_NN(n=n, cutoff_radius=rc)
+            for i, atom in enumerate(self._atoms):
+                CN = 0
+                for d in NN_d[i]:
+                    if d < rc:
+                        CN += 1
+                atom.CN = CN
 
     @property
-    def M(self):
-        """Total mass of all atoms."""
-        return math.fsum(self.masses)
+    def nearest_neighbors(self):
+        """Return array of nearest-neighbor atoms for each `Atom`."""
+        nearest_neighbors = []
+        for atom in self._atoms:
+            nearest_neighbors.append(atom.NN)
+        self._nearest_neighbors = nearest_neighbors[:]
+        return np.asarray(self._nearest_neighbors)
+
+    def update_nearest_neighbors(self, n=6, rc=np.inf):
+        if self._use_kdtree:
+            NN_d, NN_i = self.query_NN(n=n, cutoff_radius=rc)
+            for i, atom in enumerate(self._atoms):
+                NN_atoms = []
+                for j, d in enumerate(NN_d[i]):
+                    if d < rc:
+                        NN_atoms.append(self._atoms[NN_i[i][j]])
+                atom.NN = NN_atoms
+
+    def query_NN(self, n=6, eps=0, p=2, cutoff_radius=np.inf):
+        """Query nearest neighbors.
+
+        Parameters
+        ----------
+        n : integer
+            The number of nearest neighbors to return.
+        eps : nonnegative float
+            Return approximate nearest neighbors; the kth returned value
+            is guaranteed to be no further than (1+eps) times the
+            distance to the real kth nearest neighbor.
+        p : float, 1<=p<=infinity
+            Which Minkowski p-norm to use.
+            1 is the sum-of-absolute-values "Manhattan" distance
+            2 is the usual Euclidean distance
+            infinity is the maximum-coordinate-difference distance
+        cutoff_radius : nonnegative float
+            Return only neighbors within this distance. This is used to prune
+            tree searches, so if you are doing a series of nearest-neighbor
+            queries, it may help to supply the distance to the nearest neighbor
+            of the most recent point.
+
+        Returns
+        -------
+        NN_d : array of floats
+            The distances to the nearest neighbors.
+        NN_i : array of integers
+            The locations of the neighbors in self.atom_tree.data. NN_i is the
+            same shape as NN_d.
+
+        """
+        NNtree = self.atom_tree
+        NN_d = NN_i = None
+        if NNtree is not None:
+            d, i = NNtree.query(self.coords, k=n+1, eps=eps, p=p,
+                                distance_upper_bound=cutoff_radius)
+            NN_d, NN_i = d[:, 1:], i[:, 1:]
+        return NN_d, NN_i
 
     @property
     def masses(self):
-        """Return the list of Atom masses."""
+        """Return the list of `Atom` masses."""
         #self._masses_array = np.asarray(self._masses)
         masses = []
         for atom in self._atoms:
@@ -192,14 +326,29 @@ class Atoms(MutableSequence):
         self._masses = masses[:]
         return self._masses
 
+    @property
+    def m(self):
+        """Total mass of `Atoms`.
+
+        .. deprecated:: 0.1.0
+           Use :py:attr:`M` attribute instead.
+
+        """
+        return self.M
+
+    @property
+    def M(self):
+        """Total mass of `Atoms`."""
+        return math.fsum(self.masses)
+
     @property
     def Natoms(self):
-        """Number of atoms in Atoms."""
+        """Number of atoms in `Atoms`."""
         return len(self._atoms)
 
     @property
     def Ntypes(self):
-        """Number of atom types in Atoms."""
+        """Number of atom types in `Atoms`."""
         return len(self.atomtypes.keys())
 
     @property
@@ -214,7 +363,7 @@ class Atoms(MutableSequence):
 
     @property
     def symbols(self):
-        """Return array of Atom symbols."""
+        """Return array of `Atom` symbols."""
         symbols = []
         for atom in self._atoms:
             symbols.append(atom.symbol)
@@ -223,7 +372,7 @@ class Atoms(MutableSequence):
 
     @property
     def velocities(self):
-        """Return array of Atom velocities."""
+        """Return array of `Atom` velocities."""
         velocities = []
         for atom in self._atoms:
             velocities.append(atom.v)
@@ -235,8 +384,8 @@ class Atoms(MutableSequence):
 
         Parameters
         ----------
-        atom : Atom
-            an intance of an Atom object
+        atom : `Atom`
+            an intance of an `Atom` object
 
         """
         if atom.atomtype not in self._atomtypes:
@@ -249,12 +398,17 @@ class Atoms(MutableSequence):
         Parameters
         ----------
         atomtypes : sequence
-            a list of Atom object instances
+            a list of `Atom` object instances
 
         """
         for atom in atomtypes:
             self.add_atomtype(atom)
 
+    def assign_unique_ids(self, starting_id=1):
+        """Assign unique ID to each `Atom` in `Atoms`."""
+        for i, atom in enumerate(self._atoms, start=starting_id):
+            atom.atomID = i
+
     def center_CM(self, r_indices=[0, 1, 2]):
         """Center atoms on CM coordinates
 
@@ -305,7 +459,7 @@ class Atoms(MutableSequence):
                                 break
 
     def filter_atoms(self, filter_atom_ids, invert=False):
-        """Filter Atoms.
+        """Filter `Atoms`.
 
         Parameters
         ----------
@@ -314,7 +468,7 @@ class Atoms(MutableSequence):
 
         Returns
         -------
-        filtered_atoms : Atoms
+        filtered_atoms : `Atoms`
 
         """
         #if invert:
@@ -346,6 +500,28 @@ class Atoms(MutableSequence):
             smallest allowed absolute value of any component of an
             atom's position
 
+        """
+        for atom in self._atoms:
+            r = atom.r.tolist()
+            for i, ri in enumerate(r[:]):
+                if ri < 0 and abs(ri) < epsilon:
+                    r[i] = 0.0
+            atom.x, atom.y, atom.z = r
+
+    def rezero_xyz(self, epsilon=1.0e-10):
+        return self.rezero_coords(epsilon=epsilon)
+
+    def rezero_coords(self, epsilon=1.0e-10):
+        """Set really really small coordinates to zero.
+
+        Set all coordinates with absolute value less than
+        epsilon to zero.
+
+        Parameters
+        ----------
+        epsilon : float
+            smallest allowed absolute value of any :math:`x,y,z` component.
+
         """
         for atom in self._atoms:
             r = atom.r.tolist()
@@ -354,19 +530,31 @@ class Atoms(MutableSequence):
                     r[i] = 0.0
             atom.x, atom.y, atom.z = r
 
-    def get_atoms(self, asarray=False):
-        """Return list of Atoms.
+    def getatomsattr(self, asarray=False, as_array=False):
+        pass
+
+    def get_atom(self, atomID=None, index=None):
+        try:
+            return self._atoms[atomID - 1]
+        except (TypeError, IndexError):
+            try:
+                return self._atoms[index]
+            except (IndexError, TypeError):
+                return None
+
+    def get_atoms(self, asarray=False, as_array=False):
+        """Return list of `Atoms`.
 
         Parameters
         ----------
-        asarray : bool, optional
+        asarray, as_array : bool, optional
 
         Returns
         -------
         sequence or ndarray
 
         """
-        if asarray:
+        if asarray or as_array:
             return np.asarray(self._atoms)
         else:
             return self._atoms
@@ -398,7 +586,7 @@ class Atoms(MutableSequence):
             return coords
 
     def get_filtered_atom_ids(self, filter_atom_ids, invert=False):
-        """Return atom ids filtered by list of ``filter_atom_ids``.
+        """Return atom ids filtered by list of `filter_atom_ids`.
 
         Parameters
         ----------
@@ -450,12 +638,12 @@ class Atoms(MutableSequence):
             filtered_coords
 
     def rotate(self, R_matrix):
-        """Rotate atom coordinates using rotation matrix ``R_matrix``.
+        """Rotate atom coordinates using rotation matrix `R_matrix`.
 
         Parameters
         ----------
         R_matrix : array_like
-            3x3 array representation of ``3D`` rotation matrix.
+            3x3 array representation of *3D rotation matrix*.
 
         """
         for atom in self._atoms:
@@ -471,7 +659,7 @@ class Atoms(MutableSequence):
             components of atoms position vector through
         r_indices : sequence, optional
             list of component indices of position vector that
-            ``dr`` vector acts upon
+            `dr` vector acts upon
 
         """
         for atom in self._atoms:
@@ -484,9 +672,9 @@ class Atoms(MutableSequence):
     def __delitem__(self, index):
         """Concrete implementation of @abstractmethod.
 
-        delete list element ``self.atoms[index]`` and delete all elements
-        from atom properties lists ``self.masses[index]``,
-        ``self.charges[index]``, and ``self.coords[index]``
+        delete list element `self.atoms[index]` and delete all elements
+        from atom properties lists `self.masses[index]`,
+        `self.charges[index]`, and `self.coords[index]`
 
         Parameters
         ----------
@@ -501,7 +689,7 @@ class Atoms(MutableSequence):
     def __getitem__(self, index):
         """Concrete implementation of @abstractmethod.
 
-        get Atom object instance at list element ``self.atoms[index]``
+        get `Atom` object instance at list element `self.atoms[index]`
 
         Parameters
         ----------
@@ -510,8 +698,8 @@ class Atoms(MutableSequence):
 
         Returns
         -------
-        Atom
-            Atom object instance at target ``self.atoms[index]``
+        `Atom`
+            `Atom` object instance at target `self.atoms[index]`
 
         """
         return self._atoms[index]
@@ -522,7 +710,7 @@ class Atoms(MutableSequence):
         Returns
         -------
         int
-            length of ``self.atoms`` list.
+            length of `self.atoms` list.
 
         """
         return len(self._atoms)
@@ -530,19 +718,19 @@ class Atoms(MutableSequence):
     def __setitem__(self, index, atom):
         """Concrete implementation of @abstractmethod.
 
-        set target list element ``self.atoms[index] = atom``
+        set target list element `self.atoms[index] = atom`
 
-        Also set element of all atom properties lists (``self.masses[index]``,
-        ``self.charges[index]``, and ``self.coords[index]``) to atom instance
-        properties (``atom.m``, ``atom.q``, ``atom.r``), respectively.
+        Also set element of all atom properties lists (`self.masses[index]`,
+        `self.charges[index]`, and `self.coords[index]`) to atom instance
+        properties (`atom.m`, `atom.q`, `atom.r`), respectively.
 
 
         Parameters
         ----------
         index : int
             index of target list element
-        atom : Atom
-            Atom object instance to set target list element to
+        atom : `Atom`
+            `Atom` instance object to set target list element to.
 
         """
         self._check_type(atom)
@@ -553,20 +741,20 @@ class Atoms(MutableSequence):
     def insert(self, index, atom):
         """Concrete implementation of @abstractmethod.
 
-        insert Atom instance at target list ``index``
+        insert `Atom` instance at target list `index`
 
-        Also insert Atom instance properties at the given target list index
-        for all Atom properties in ``self._property_lists.keys()``
-        into their respective target lists of Atoms properties
-        ``self._property_lists.values()``.
+        Also insert `Atom` instance properties at the given target list index
+        for all `Atom` properties in `self._property_lists.keys()`
+        into their respective target lists of `Atoms` properties
+        `self._property_lists.values()`.
 
 
         Parameters
         ----------
         index : int
             index of target list element
-        atom : Atom
-            Atom object instance to set target list element to
+        atom : `Atom`
+            `Atom` object instance to set target list element to
 
         """
         self._check_type(atom)
