@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 """
-=====================================================================
-Nanotube structure tools (:mod:`sknano.nanogen._nanotube_generator`)
-=====================================================================
+==========================================================================
+Nanotube structure generators (:mod:`sknano.nanogen._nanotube_generator`)
+==========================================================================
 
 .. currentmodule:: sknano.nanogen._nanotube_generator
 
@@ -49,19 +49,15 @@ from ..structure_io import DATAWriter, XYZWriter, default_structure_format, \
     supported_structure_formats
 
 from ._nanotube import Nanotube, NanotubeBundle
+from ._structure_generator import StructureGenerator, StructureGeneratorError
 
-__all__ = ['NanotubeGeneratorError',
-           'NanotubeGenerator',
+__all__ = ['NanotubeGenerator',
            'NanotubeBundleGenerator',
-           'MWNTGenerator']
+           'MWNTGenerator',
+           'NanotubeGeneratorError']
 
 
-class NanotubeGeneratorError(Exception):
-    """Base class for NanotubeGenerator exceptions."""
-    pass
-
-
-class NanotubeGenerator(Nanotube):
+class NanotubeGenerator(Nanotube, StructureGenerator):
     u"""Class for generating nanotube structures.
 
     Parameters
@@ -79,30 +75,30 @@ class NanotubeGenerator(Nanotube):
         nearest neighbor atoms. Must be in units of **Angstroms**.
     Lz : float, optional
         Length of nanotube in units of **nanometers**.
-        Overrides the ``nz`` value.
+        Overrides the `nz` value.
 
         .. versionadded:: 0.2.5
 
     tube_length : float, optional
         Length of nanotube in units of **nanometers**.
-        Overrides the ``nz`` value.
+        Overrides the `nz` value.
 
         .. deprecated:: 0.2.5
-           Use ``Lz`` instead
+           Use `Lz` instead
 
     fix_Lz : bool, optional
         Generate the nanotube with length as close to the specified
-        :math:`L_z` as possible. If ``True``, then
+        :math:`L_z` as possible. If `True`, then
         non integer :math:`n_z` cells are permitted.
 
         .. versionadded:: 0.2.6
 
     autogen : bool, optional
-        if ``True``, automatically call
+        if `True`, automatically call
         :py:meth:`~NanotubeGenerator.generate_unit_cell`,
         followed by :py:meth:`~NanotubeGenerator.generate_structure_data`.
     verbose : bool, optional
-        if ``True``, show verbose output
+        if `True`, show verbose output
 
     Examples
     --------
@@ -141,19 +137,10 @@ class NanotubeGenerator(Nanotube):
             bond=bond, Lx=Lx, Ly=Ly, Lz=Lz, fix_Lz=fix_Lz,
             with_units=False, verbose=verbose)
 
-        self._fname = None
-        self.unit_cell = None
-        self.structure_atoms = None
-
         if autogen:
             self.generate_unit_cell()
             self.generate_structure_data()
 
-    @property
-    def fname(self):
-        """Structure file name."""
-        return self._fname
-
     def generate_unit_cell(self):
         """Generate the nanotube unit cell."""
         eps = 0.01
@@ -180,7 +167,7 @@ class NanotubeGenerator(Nanotube):
             print('dpsi: {}'.format(dpsi))
             print('dtau: {}\n'.format(dtau))
 
-        self.unit_cell = Atoms()
+        self._unit_cell = Atoms()
 
         for i in xrange(1, N + 1):
             x1 = rt * np.cos(i * psi)
@@ -196,7 +183,7 @@ class NanotubeGenerator(Nanotube):
             if verbose:
                 print('Basis Atom 1:\n{}'.format(atom1))
 
-            self.unit_cell.append(atom1)
+            self._unit_cell.append(atom1)
 
             x2 = rt * np.cos(i * psi + dpsi)
             y2 = rt * np.sin(i * psi + dpsi)
@@ -210,17 +197,18 @@ class NanotubeGenerator(Nanotube):
             if verbose:
                 print('Basis Atom 2:\n{}'.format(atom2))
 
-            self.unit_cell.append(atom2)
+            self._unit_cell.append(atom2)
 
     def generate_structure_data(self):
         """Generate structure data."""
-        self.structure_atoms = []
+        self._structure_atoms = Atoms()
         for nz in xrange(int(np.ceil(self._nz))):
             dr = np.array([0.0, 0.0, nz * self.T])
-            for uc_atom in self.unit_cell.atoms:
+            for uc_atom in self._unit_cell:
                 nt_atom = Atom(uc_atom.symbol)
                 nt_atom.r = uc_atom.r + dr
-                self.structure_atoms.append(nt_atom)
+                self._structure_atoms.append(nt_atom)
+        #self.update_structure_atoms_attributes()
 
     def save_data(self, fname=None, structure_format=None,
                   rotation_angle=None, rot_axis=None, deg2rad=True,
@@ -237,14 +225,14 @@ class NanotubeGenerator(Nanotube):
                 - xyz
                 - data
 
-            If ``None``, then guess based on ``fname`` file extension or
-            default to ``xyz`` format.
+            If `None`, then guess based on `fname` file extension or
+            default to `xyz` format.
         rotation_angle : {None, float}, optional
             Angle of rotation
         rot_axis : {'x', 'y', 'z'}, optional
             Rotation axis
         deg2rad : bool, optional
-            Convert ``rotation_angle`` from degrees to radians.
+            Convert `rotation_angle` from degrees to radians.
         center_CM : bool, optional
             Center center-of-mass on origin.
 
@@ -280,32 +268,25 @@ class NanotubeGenerator(Nanotube):
                         structure_format not in supported_structure_formats:
                     structure_format = default_structure_format
 
-        #structure_atoms = list(itertools.chain(*self.structure_atoms))
-        structure_atoms = None
-        if isinstance(self.structure_atoms, list):
-            structure_atoms = Atoms(self.structure_atoms)
-        elif isinstance(self.structure_atoms, Atoms):
-            structure_atoms = self.structure_atoms
+        self._fname = fname
 
         if center_CM:
-            structure_atoms.center_CM()
+            self._structure_atoms.center_CM()
 
         if self._L0 is not None and self._fix_Lz:
-            structure_atoms.clip_bounds(abs_limit=(10 * self._L0 + 0.5) / 2,
-                                        r_indices=[2])
+            self._structure_atoms.clip_bounds(
+                abs_limit=(10 * self._L0 + 0.5) / 2, r_indices=[2])
 
         if rotation_angle is not None:
             R_matrix = rotation_matrix(rotation_angle,
                                        rot_axis=rot_axis,
                                        deg2rad=deg2rad)
-            structure_atoms.rotate(R_matrix)
+            self._structure_atoms.rotate(R_matrix)
 
         if structure_format == 'data':
-            DATAWriter.write(fname=fname, atoms=structure_atoms)
+            DATAWriter.write(fname=self._fname, atoms=self._structure_atoms)
         else:
-            XYZWriter.write(fname=fname, atoms=structure_atoms)
-
-        self._fname = fname
+            XYZWriter.write(fname=self._fname, atoms=self._structure_atoms)
 
 
 class NanotubeBundleGenerator(NanotubeGenerator, NanotubeBundle):
@@ -348,17 +329,17 @@ class NanotubeBundleGenerator(NanotubeGenerator, NanotubeBundle):
 
     fix_Lz : bool, optional
         Generate the nanotube with length as close to the specified
-        :math:`L_z` as possible. If ``True``, then
+        :math:`L_z` as possible. If `True`, then
         non integer :math:`n_z` cells are permitted.
 
         .. versionadded:: 0.2.6
 
     autogen : bool, optional
-        if ``True``, automatically call
+        if `True`, automatically call
         :py:meth:`~NanotubeGenerator.generate_unit_cell`,
         followed by :py:meth:`~NanotubeGenerator.generate_structure_data`.
     verbose : bool, optional
-        if ``True``, show verbose output
+        if `True`, show verbose output
 
     Examples
     --------
@@ -458,6 +439,8 @@ class NanotubeBundleGenerator(NanotubeGenerator, NanotubeBundle):
             self._r2[0] = self._r1[0] * np.cos(2 * np.pi / 3)
             self._r2[1] = self._r1[0] * np.sin(2 * np.pi / 3)
 
+        #self._r1[2] = self._r2[2] = self.T
+
         self._bundle_packing = bundle_packing
         self._bundle_geometry = bundle_geometry
 
@@ -471,8 +454,8 @@ class NanotubeBundleGenerator(NanotubeGenerator, NanotubeBundle):
 
         self._Ntubes = 0
 
-        swcnt0 = copy.deepcopy(self.structure_atoms)
-        self.structure_atoms = Atoms()
+        swnt0 = copy.deepcopy(self._structure_atoms)
+        self._structure_atoms = Atoms()
         if self._bundle_geometry == 'hexagon':
             nrows = max(self._nx, self._ny, 3)
             if nrows % 2 != 1:
@@ -485,39 +468,40 @@ class NanotubeBundleGenerator(NanotubeGenerator, NanotubeBundle):
             while ntubes_per_row >= ntubes_per_end_rows:
                 if row == 0:
                     for n in xrange(ntubes_per_row):
-                        swcnt = Atoms(atoms=swcnt0, deepcopy=True)
-                        swcnt.center_CM()
+                        swnt = Atoms(atoms=swnt0, deepcopy=True)
+                        swnt.center_CM()
                         dr = n * self._r1
-                        swcnt.translate(dr)
-                        self.structure_atoms.extend(swcnt.atoms)
+                        swnt.translate(dr)
+                        self._structure_atoms.extend(swnt.atoms)
                         self._Ntubes += 1
                 else:
                     for nx in xrange(1, ntubes_per_row + 1):
                         for ny in (-row, row):
-                            swcnt = Atoms(atoms=swcnt0, deepcopy=True)
-                            swcnt.center_CM()
+                            swnt = Atoms(atoms=swnt0, deepcopy=True)
+                            swnt.center_CM()
                             dy = np.zeros(3)
                             dy[0] = abs(ny) * self._r2[0]
                             dy[1] = ny * self._r2[1]
                             dr = nx * self._r1 + dy
-                            swcnt.translate(dr)
-                            self.structure_atoms.extend(swcnt.atoms)
+                            swnt.translate(dr)
+                            self._structure_atoms.extend(swnt.atoms)
                             self._Ntubes += 1
                 row += 1
                 ntubes_per_row = nrows - row
         else:
             for nx in xrange(self._nx):
                 for ny in xrange(self._ny):
-                    swcnt = Atoms(atoms=swcnt0, deepcopy=True)
-                    swcnt.center_CM()
+                    swnt = Atoms(atoms=swnt0, deepcopy=True)
+                    swnt.center_CM()
                     dr = nx * self._r1 + ny * self._r2
-                    swcnt.translate(dr)
-                    self.structure_atoms.extend(swcnt.atoms)
+                    swnt.translate(dr)
+                    self._structure_atoms.extend(swnt.atoms)
                     self._Ntubes += 1
         self._Natoms_per_bundle = \
             self.compute_Natoms_per_bundle(n=self._n, m=self._m,
                                            nz=self._nz,
                                            Ntubes=self._Ntubes)
+        #self.update_structure_atoms_attributes()
 
     def save_data(self, fname=None, structure_format=None,
                   rotation_angle=None, rot_axis=None, deg2rad=True,
@@ -534,14 +518,14 @@ class NanotubeBundleGenerator(NanotubeGenerator, NanotubeBundle):
                 - xyz
                 - data
 
-            If ``None``, then guess based on ``fname`` file extension or
-            default to ``xyz`` format.
+            If `None`, then guess based on `fname` file extension or
+            default to `xyz` format.
         rotation_angle : {None, float}, optional
             Angle of rotation
         rot_axis : {'x', 'y', 'z'}, optional
             Rotation axis
         deg2rad : bool, optional
-            Convert ``rotation_angle`` from degrees to radians.
+            Convert `rotation_angle` from degrees to radians.
         center_CM : bool, optional
             Center center-of-mass on origin.
 
@@ -679,8 +663,6 @@ class MWNTGenerator(NanotubeGenerator, NanotubeBundle):
             element2=element2, Lx=Lx, Ly=Ly, Lz=Lz, fix_Lz=fix_Lz, bond=bond,
             autogen=False, verbose=verbose)
 
-        self._Lzmin = np.inf
-
         self._max_shells = max_shells
         if max_shells is None:
             self._max_shells = np.inf
@@ -724,7 +706,7 @@ class MWNTGenerator(NanotubeGenerator, NanotubeBundle):
             super(MWNTGenerator, self).generate_unit_cell()
             self.generate_structure_data()
 
-    def generate_unit_cell(self, n=int, m=int):
+    def _generate_unit_cell(self, n=int, m=int):
         """Generate the unit cell of a MWNT shell"""
         eps = 0.01
         bond = self._bond
@@ -772,25 +754,27 @@ class MWNTGenerator(NanotubeGenerator, NanotubeBundle):
 
     def generate_structure_data(self):
         """Generate structure data."""
-        super(MWNTGenerator, self).generate_structure_data()
-
-        self._Ntubes = 0
-
         dt = []
         Ch = []
         for n in xrange(0, 201):
             for m in xrange(0, 201):
-                dt.append(Nanotube.compute_dt(n=n, m=m, bond=self.bond))
+                dt.append(Nanotube.compute_dt(n=n, m=m, bond=self._bond))
                 Ch.append((n, m))
         dt = np.asarray(dt)
         Ch = np.asarray(Ch)
 
-        swnt0 = copy.deepcopy(self.structure_atoms)
+        super(MWNTGenerator, self).generate_structure_data()
+
+        self._Ntubes = 0
+
+        swnt0 = copy.deepcopy(self._structure_atoms)
+        self._structure_atoms = Atoms()
+
         mwnt0 = Atoms(atoms=swnt0, deepcopy=True)
-        self._Lzmin = min(self._Lzmin, self._Lz)
         mwnt0.center_CM()
 
-        next_dt = self.dt - 2 * self._shell_spacing
+        Lzmin = self._Lz
+        next_dt = self._dt - 2 * self._shell_spacing
         while next_dt >= self._min_shell_diameter and \
                 self._Nshells_per_tube < self._max_shells:
             # get chiral indices for next_dt
@@ -834,41 +818,43 @@ class MWNTGenerator(NanotubeGenerator, NanotubeBundle):
 
                 #delta_dt += 0.05
 
-            n, m = \
-                next_Ch_candidates[
-                    np.random.choice(np.arange(len(next_Ch_candidates)))]
-            # generate unit cell for new chiral indices
-            unit_cell = self.generate_unit_cell(n=n, m=m)
+            n, m = next_Ch_candidates[
+                np.random.choice(np.arange(len(next_Ch_candidates)))]
+            T = Nanotube.compute_T(n=n, m=m, bond=self._bond)
+            Lz = Nanotube.compute_Lz(n=n, m=m, bond=self._bond, nz=self._nz)
+            Lzmin = min(Lzmin, Lz)
+
+            # generate unit cell for new shell chiral indices
+            shell_unit_cell = self._generate_unit_cell(n=n, m=m)
+
             if self._verbose:
-                print('next_dt: {:.4f}'.format(next_dt))
-                print('n, m = {}, {}'.format(n, m))
-                print('unit_cell.Natoms: {}\n'.format(unit_cell.Natoms))
-            T = Nanotube.compute_T(n=n, m=m, bond=self.bond)
-            Lz = Nanotube.compute_Lz(n=n, m=m, bond=self.bond, nz=self.nz)
-            self._Lzmin = min(self._Lzmin, Lz)
-            shell_atoms = Atoms()
+                print('new shell:\n'
+                      'n, m = {}, {}\n'.format(n, m) +
+                      'dt: {:.4f}\n'.format(next_dt) +
+                      'shell_unit_cell.Natoms: {}\n'.format(
+                          shell_unit_cell.Natoms))
+
+            shell = Atoms()
             for nz in xrange(int(np.ceil(self._nz))):
                 dr = np.array([0.0, 0.0, nz * T])
-                for uc_atom in unit_cell.atoms:
-                    nt_atom = Atom(uc_atom.symbol)
-                    nt_atom.r = uc_atom.r + dr
-                    shell_atoms.append(nt_atom)
-            shell_atoms.center_CM()
-            mwnt0.extend(shell_atoms.atoms)
-            next_dt -= 2 * self._shell_spacing
+                for uc_atom in shell_unit_cell:
+                    atom = Atom(uc_atom.symbol)
+                    atom.r = uc_atom.r + dr
+                    shell.append(atom)
+            shell.center_CM()
+            mwnt0.extend(shell.atoms)
             self._Nshells_per_tube += 1
+            next_dt -= 2 * self._shell_spacing
 
         if self._L0 is not None and self._fix_Lz:
-            mwnt0.clip_bounds(abs_limit=(10 * self._L0 + 0.5) / 2,
-                              r_indices=[2])
+            mwnt0.clip_bounds(
+                abs_limit=(10 * self._L0 + 0.5) / 2, r_indices=[2])
         else:
-            mwnt0.clip_bounds(abs_limit=(10 * self._Lzmin + 0.5) / 2,
-                              r_indices=[2])
+            mwnt0.clip_bounds(
+                abs_limit=(10 * Lzmin + 0.5) / 2, r_indices=[2])
 
         self._Natoms_per_tube = mwnt0.Natoms
 
-        self.structure_atoms = Atoms()
-
         if self._bundle_geometry == 'hexagon':
             nrows = max(self._nx, self._ny, 3)
             if nrows % 2 != 1:
@@ -885,7 +871,7 @@ class MWNTGenerator(NanotubeGenerator, NanotubeBundle):
                         mwnt.center_CM()
                         dr = n * self._r1
                         mwnt.translate(dr)
-                        self.structure_atoms.extend(mwnt.atoms)
+                        self._structure_atoms.extend(mwnt.atoms)
                         self._Ntubes += 1
                 else:
                     for nx in xrange(1, ntubes_per_row + 1):
@@ -897,7 +883,7 @@ class MWNTGenerator(NanotubeGenerator, NanotubeBundle):
                             dy[1] = ny * self._r2[1]
                             dr = nx * self._r1 + dy
                             mwnt.translate(dr)
-                            self.structure_atoms.extend(mwnt.atoms)
+                            self._structure_atoms.extend(mwnt.atoms)
                             self._Ntubes += 1
                 row += 1
                 ntubes_per_row = nrows - row
@@ -908,9 +894,10 @@ class MWNTGenerator(NanotubeGenerator, NanotubeBundle):
                     mwnt.center_CM()
                     dr = nx * self._r1 + ny * self._r2
                     mwnt.translate(dr)
-                    self.structure_atoms.extend(mwnt.atoms)
+                    self._structure_atoms.extend(mwnt.atoms)
                     self._Ntubes += 1
         self._Natoms_per_bundle = self._Ntubes * self._Natoms_per_tube
+        #self.update_structure_atoms_attributes()
 
         if self._verbose:
             print('Ntubes: {}'.format(self._Ntubes))
@@ -933,14 +920,14 @@ class MWNTGenerator(NanotubeGenerator, NanotubeBundle):
                 - xyz
                 - data
 
-            If ``None``, then guess based on ``fname`` file extension or
-            default to ``xyz`` format.
+            If `None`, then guess based on `fname` file extension or
+            default to `xyz` format.
         rotation_angle : {None, float}, optional
             Angle of rotation
         rot_axis : {'x', 'y', 'z'}, optional
             Rotation axis
         deg2rad : bool, optional
-            Convert ``rotation_angle`` from degrees to radians.
+            Convert `rotation_angle` from degrees to radians.
         center_CM : bool, optional
             Center center-of-mass on origin.
 
@@ -1003,3 +990,8 @@ class MWNTGenerator(NanotubeGenerator, NanotubeBundle):
             fname=fname, structure_format=structure_format,
             rotation_angle=rotation_angle, rot_axis=rot_axis,
             deg2rad=deg2rad, center_CM=center_CM)
+
+
+class NanotubeGeneratorError(StructureGeneratorError):
+    """Base class for NanotubeGenerator exceptions."""
+    pass
