@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 """
-===============================================================================
-scikit-nano command-line scripts (:mod:`sknano.scripts`)
-===============================================================================
+=============================================
+Command-line scripts (:mod:`sknano.scripts`)
+=============================================
 
 .. currentmodule:: sknano.scripts
 
