@@ -267,7 +267,7 @@ class DUMPWriter(StructureWriter):
             if comment_line is None:
                 comment_line = fname
 
-            atoms.fix_minus_zero_coords()
+            atoms.rezero_coords()
 
             atomtypes = atoms.atomtypes
 
